// Copyright 2023 The Pigweed Authors
//
// Licensed under the Apache License, Version 2.0 (the "License"); you may not
// use this file except in compliance with the License. You may obtain a copy of
// the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations under
// the License.

#pragma once
#include "chre/platform/condition_variable.h"
#include "pw_chrono/system_clock.h"

namespace chre {

inline ConditionVariable::ConditionVariable() {}

inline ConditionVariable::~ConditionVariable() {}

inline void ConditionVariable::notify_one() { notification_.release(); }

inline void ConditionVariable::wait(Mutex& mutex) {
  mutex.unlock();
  notification_.acquire();
  mutex.lock();
}

inline bool ConditionVariable::wait_for(Mutex& mutex, Nanoseconds timeout) {
  mutex.unlock();
  auto pw_timeout = pw::chrono::SystemClock::for_at_least(
      std::chrono::nanoseconds(timeout.toRawNanoseconds()));
  bool did_acquire = notification_.try_acquire_for(pw_timeout);
  mutex.lock();
  return did_acquire;
}

}  // namespace chre
